// src/checkbox.ts
import { createProvideScope } from "@oku-ui/provide";
import { Transition, defineComponent, h, onMounted, ref, toRefs, watch, watchEffect } from "vue";
import { composeEventHandlers } from "@oku-ui/utils";
import { useControllableRef, usePrevious, useRef, useSize } from "@oku-ui/use-composable";
import { Primitive } from "@oku-ui/primitive";
function isIndeterminate(checked) {
  return checked === "indeterminate";
}
function getState(checked) {
  return isIndeterminate(checked) ? "indeterminate" : checked ? "checked" : "unchecked";
}
var BubbleInput = defineComponent({
  name: "BubbleInput",
  inheritAttrs: false,
  props: {
    checked: {
      type: [Boolean, "indeterminate"],
      default: false
    },
    control: {
      type: Object,
      default: null
    },
    bubbles: {
      type: Boolean,
      default: true
    }
  },
  setup(props, { attrs }) {
    const { ...inputAttrs } = attrs;
    const { checked, control, bubbles } = props;
    const _ref = ref();
    const prevChecked = usePrevious(checked);
    const controlSize = useSize(control);
    watchEffect(() => {
      const input = _ref.value;
      const inputProto = window.HTMLInputElement.prototype;
      const descriptor = Object.getOwnPropertyDescriptor(inputProto, "checked");
      const setChecked = descriptor.set;
      if (prevChecked !== checked && setChecked) {
        const event = new Event("click", { bubbles });
        input.indeterminate = isIndeterminate(checked);
        setChecked.call(input, isIndeterminate(checked) ? false : checked);
        input.dispatchEvent(event);
      }
    });
    return () => h("input", {
      "type": "checkbox",
      "aria-hidden": true,
      "defaultChecked": isIndeterminate(checked) ? false : checked,
      ...inputAttrs,
      "tabIndex": -1,
      "ref": _ref,
      "style": {
        ...inputAttrs.style,
        ...controlSize,
        position: "absolute",
        pointerEvents: "none",
        opacity: 0,
        margin: 0
      }
    });
  }
});
var CHECKBOX_NAME = "Checkbox";
var [createCheckboxProvider, _createCheckboxScope] = createProvideScope(CHECKBOX_NAME);
var [CheckboxProvider, useCheckboxInject] = createCheckboxProvider(CHECKBOX_NAME);
var checkboxDisplayName = "OkuCheckbox";
var Checkbox = defineComponent({
  name: checkboxDisplayName,
  components: { BubbleInput },
  inheritAttrs: false,
  props: {
    checked: {
      type: [Boolean, "indeterminate"],
      default: void 0
    },
    defaultChecked: {
      type: [Boolean, "indeterminate"],
      default: void 0
    },
    required: {
      type: Boolean,
      default: void 0
    },
    onCheckedChange: Function,
    scopeCheckbox: {
      type: Object,
      required: false,
      default: void 0
    }
  },
  setup(props, { attrs, slots, expose }) {
    const { checked: checkedProp, scopeCheckbox, defaultChecked, onCheckedChange, required } = toRefs(props);
    const { _ref: buttonRef, refEl: buttonRefEl } = useRef();
    expose({
      innerRef: buttonRefEl
    });
    const {
      name,
      disabled,
      value = "on",
      ...checkboxProps
    } = attrs;
    const hasConsumerStoppedPropagationRef = ref(false);
    const isFormControl = buttonRefEl.value ? Boolean(buttonRefEl.value.closest("form")) : true;
    const { state } = useControllableRef({
      prop: checkedProp.value,
      defaultProp: defaultChecked.value,
      onChange: onCheckedChange.value
    });
    const initialCheckedStateRef = ref();
    onMounted(() => {
      initialCheckedStateRef.value = state.value;
    });
    watch([buttonRefEl, state], () => {
      var _a;
      const form = (_a = buttonRefEl.value) == null ? void 0 : _a.form;
      if (form) {
        const reset = () => state.value = initialCheckedStateRef.value;
        form.addEventListener("reset", reset);
        return () => form.removeEventListener("reset", reset);
      }
    });
    CheckboxProvider({
      scope: scopeCheckbox.value,
      state,
      disabled
    });
    const originalReturn = () => [
      h(
        Primitive.button,
        {
          "type": "button",
          "role": "checkbox",
          "aria-checked": isIndeterminate(state.value) ? "mixed" : state.value,
          "aria-required": required.value,
          "data-state": getState(state.value),
          "data-disabled": disabled ? "" : void 0,
          "disabled": disabled,
          "value": value,
          ...checkboxProps,
          "ref": buttonRef,
          "onKeyDown": composeEventHandlers(checkboxProps.onKeydown, (event) => {
            if (event.key === "Enter")
              event.preventDefault();
          }),
          "onClick": composeEventHandlers(checkboxProps.onClick, (event) => {
            state.value = isIndeterminate(state.value) ? true : !state.value;
            if (isFormControl) {
              if (!hasConsumerStoppedPropagationRef.value)
                event.stopPropagation();
            }
          })
        },
        {
          default: () => {
            var _a;
            return (_a = slots.default) == null ? void 0 : _a.call(slots);
          }
        }
      ),
      isFormControl && h(
        BubbleInput,
        {
          control: buttonRefEl.value,
          bubbles: !hasConsumerStoppedPropagationRef.value,
          name,
          value,
          checked: state.value,
          required,
          disabled,
          // We transform because the input is absolutely positioned but we have
          // rendered it **after** the button. This pulls it back to sit on top
          // of the button.
          style: { transform: "translateX(-100%)" }
        }
      )
    ];
    return originalReturn;
  }
});
var INDICATOR_NAME = "CheckboxIndicator";
var CheckboxIndicator = defineComponent({
  name: "CheckboxIndicator",
  components: { Transition },
  props: {
    scopeCheckbox: {
      type: Object,
      required: false
    },
    forceMount: Boolean
  },
  setup(props, { attrs, expose, slots }) {
    const { scopeCheckbox, forceMount } = props;
    const { ...indicatorProps } = attrs;
    const innerRef = ref();
    expose({
      innerRef
    });
    const context = useCheckboxInject(INDICATOR_NAME, scopeCheckbox);
    const originalReturn = () => h(Transition, [
      forceMount || isIndeterminate(context.value.state.value) || context.value.state.value === true ? h(
        Primitive.span,
        {
          "data-state": getState(context.value.state.value),
          "data-disabled": context.value.disabled ? "" : void 0,
          ...indicatorProps,
          "style": { pointerEvents: "none", ...attrs.style }
        },
        {
          default: () => {
            var _a;
            return (_a = slots.default) == null ? void 0 : _a.call(slots);
          }
        }
      ) : null
    ]);
    return originalReturn;
  }
});
var OkuCheckbox = Checkbox;
var OkuCheckboxIndicator = CheckboxIndicator;
export {
  OkuCheckbox,
  OkuCheckboxIndicator
};
